@@ -6,7 +6,7 @@
  *	Jeremy Lainé
  *
  * Source:
- *	http://code.google.com/p/qxmpp
+ *	https://github.com/qxmpp-project/qxmpp
  *
  * This file is a part of QXmpp library.
  *
@@ -33,7 +33,7 @@
 int main(int argc, char *argv[])
 {
     QCoreApplication a(argc, argv);
-    
+
     QXmppLogger::getLogger()->setLoggingType(QXmppLogger::StdoutLogging);
 
     QXmppClient client;
