@@ -5,7 +5,7 @@
  *	Ian Reinhart Geiser
  *
  * Source:
- *	http://code.google.com/p/qxmpp
+ *	https://github.com/qxmpp-project/qxmpp
  *
  * This file is a part of QXmpp library.
  *
@@ -29,7 +29,7 @@
 int main(int argc, char *argv[])
 {
     QCoreApplication a(argc, argv);
-    
+
     QXmppLogger::getLogger()->setLoggingType(QXmppLogger::StdoutLogging);
 
     rpcClient client;
