@@ -5,7 +5,7 @@
  *  Manjeet Dahiya
  *
  * Source:
- *  http://code.google.com/p/qxmpp
+ *  https://github.com/qxmpp-project/qxmpp
  *
  * This file is a part of QXmpp library.
  *
@@ -60,8 +60,18 @@ public:
         Paused,     ///< User had been composing but now has stopped.
     };
 
-    QXmppMessage(const QString& from = "", const QString& to = "",
-                 const QString& body = "", const QString& thread = "");
+    /// This enum describes a chat marker as defined by
+    /// XEP-0333 : Char Markers
+    enum Marker {
+        NoMarker = 0,
+        Received,
+        Displayed,
+        Acknowledged
+    };
+
+    QXmppMessage(const QString& from = QString(), const QString& to = QString(),
+                 const QString& body = QString(), const QString& thread = QString());
+
     QXmppMessage(const QXmppMessage &other);
     ~QXmppMessage();
 
@@ -106,6 +116,19 @@ public:
     QString xhtml() const;
     void setXhtml(const QString &xhtml);
 
+    // XEP-0333
+    bool isMarkable() const;
+    void setMarkable(const bool);
+
+    QString markedId() const;
+    void setMarkerId(const QString&);
+
+    QString markedThread() const;
+    void setMarkedThread(const QString&);
+
+    Marker marker() const;
+    void setMarker(const Marker);
+
     /// \cond
     void parse(const QDomElement &element);
     void toXml(QXmlStreamWriter *writer) const;
