@@ -5,7 +5,7 @@
  *  Manjeet Dahiya
  *
  * Source:
- *  http://code.google.com/p/qxmpp
+ *  https://github.com/qxmpp-project/qxmpp
  *
  * This file is a part of QXmpp library.
  *
@@ -94,47 +94,6 @@ int QXmppClientPrivate::getNextReconnectTime() const
         return 60 * 1000;
 }
 
-/// \mainpage
-///
-/// QXmpp is a cross-platform C++ XMPP client library based on the Qt
-/// framework. It tries to use Qt's programming conventions in order to ease
-/// the learning curve for new programmers.
-///
-/// QXmpp based clients are built using QXmppClient instances which handle the
-/// establishment of the XMPP connection and provide a number of high-level
-/// "managers" to perform specific tasks. You can write your own managers to
-/// extend QXmpp by subclassing QXmppClientExtension.
-///
-/// <B>Main Class:</B>
-/// - QXmppClient
-///
-/// <B>Managers to perform specific tasks:</B>
-/// - QXmppRosterManager
-/// - QXmppVCardManager
-/// - QXmppTransferManager
-/// - QXmppMucManager
-/// - QXmppCallManager
-/// - QXmppArchiveManager
-/// - QXmppVersionManager
-/// - QXmppDiscoveryManager
-/// - QXmppEntityTimeManager
-///
-/// <B>XMPP stanzas:</B> If you are interested in a more low-level API, you can refer to these
-/// classes.
-/// - QXmppIq
-/// - QXmppMessage
-/// - QXmppPresence
-///
-/// <BR><BR>
-/// <B>Project Details:</B>
-///
-/// Project Page: http://code.google.com/p/qxmpp/
-/// <BR>
-/// Report Issues: http://code.google.com/p/qxmpp/issues/
-/// <BR>
-/// New Releases: http://code.google.com/p/qxmpp/downloads/
-///
-
 /// Creates a QXmppClient object.
 /// \param parent is passed to the QObject's constructor.
 /// The default value is 0.
@@ -456,6 +415,14 @@ QAbstractSocket::SocketError QXmppClient::socketError()
     return d->stream->socket()->error();
 }
 
+/// Returns the human-readable description of the last socket error if error() is QXmppClient::SocketError.
+///
+
+QString QXmppClient::socketErrorString() const
+{
+    return d->stream->socket()->errorString();
+}
+
 /// Returns the XMPP stream error if QXmppClient::Error is QXmppClient::XmppStreamError.
 ///
 
